MODULE HPF_PREFIX_L

  USE Precision

CONTAINS
FUNCTION all_prefix_l1 (mask, segment, exclusive)
  ! ..Use Definitions..
  USE hpf_func
  ! ..Implicit none ..
  IMPLICIT NONE
  ! ..Argument definitions..
  LOGICAL, INTENT(in) :: mask(:)
  LOGICAL, INTENT(in), OPTIONAL :: segment(:)
  LOGICAL, INTENT(in), OPTIONAL :: exclusive
  LOGICAL, DIMENSION(SIZE(mask)) :: all_prefix_l1
  all_prefix_l1 = prefix_l1 (hpf_and_l1, mask, segment, exclusive, .true.)

  RETURN
END FUNCTION all_prefix_l1
FUNCTION all_prefix_l2 (mask, dim, segment, exclusive)
  ! ..Use Definitions..
  USE hpf_func
  ! ..Implicit none ..
  IMPLICIT NONE
  ! ..Argument definitions..
  LOGICAL, INTENT(in) :: mask(:,:)
  INTEGER, INTENT(in), OPTIONAL :: dim
  LOGICAL, INTENT(in), OPTIONAL :: segment(:,:)
  LOGICAL, INTENT(in), OPTIONAL :: exclusive
  LOGICAL, DIMENSION(SIZE(mask,1),SIZE(mask,2)) :: all_prefix_l2
  ! ..Local Variables..
  INTEGER :: n, n1, n2

  ! ..Executable Statements..
  IF (PRESENT(dim)) THEN
     all_prefix_l2 = prefix_l2(hpf_and_l2, mask, dim, segment, &
          exclusive, identity = .true.)
  ELSE IF (PRESENT(segment)) THEN
     n = SIZE(mask)
     n1 = SIZE(mask,1)
     n2 = SIZE(mask,2)
     all_prefix_l2 = RESHAPE(prefix_l1(hpf_and_l1, &
          mask      = RESHAPE(mask, (/n/)), &
          segment   = RESHAPE(segment,(/n/)), &
          exclusive = exclusive, &
          identity   = .true.), (/n1,n2/))
  ELSE 
     n = SIZE(mask)
     n1 = SIZE(mask,1)
     n2 = SIZE(mask,2)
     all_prefix_l2 = RESHAPE(prefix_l1(hpf_and_l1, &
          mask      = RESHAPE(mask, (/n/)), &
          exclusive = exclusive, &
          identity   = .true.), (/n1,n2/))
  END IF

  RETURN
END FUNCTION all_prefix_l2
FUNCTION any_prefix_l1 (mask, segment, exclusive)
  ! ..Use Definitions..
  USE hpf_func
  ! ..Implicit none ..
  IMPLICIT NONE
  ! ..Argument definitions..
  LOGICAL, INTENT(in) :: mask(:)
  LOGICAL, INTENT(in), OPTIONAL :: segment(:)
  LOGICAL, INTENT(in), OPTIONAL :: exclusive
  LOGICAL, DIMENSION(SIZE(mask)) :: any_prefix_l1
  any_prefix_l1 = prefix_l1 (hpf_or_l1, mask, segment, exclusive)

  RETURN
END FUNCTION any_prefix_l1
FUNCTION any_prefix_l2 (mask, dim, segment, exclusive)
  ! ..Use Definitions..
  USE hpf_func
  ! ..Implicit none ..
  IMPLICIT NONE
  ! ..Argument definitions..
  LOGICAL, INTENT(in) :: mask(:,:)
  INTEGER, INTENT(in), OPTIONAL :: dim
  LOGICAL, INTENT(in), OPTIONAL :: segment(:,:)
  LOGICAL, INTENT(in), OPTIONAL :: exclusive
  LOGICAL, DIMENSION(SIZE(mask,1),SIZE(mask,2)) :: any_prefix_l2
  ! ..Local Variables..
  INTEGER :: n, n1, n2

  ! ..Executable Statements..
  IF (PRESENT(dim)) THEN
     any_prefix_l2 = prefix_l2(hpf_or_l2, mask, dim, segment, &
          exclusive)
  ELSE IF (PRESENT(segment)) THEN
     n = SIZE(mask)
     n1 = SIZE(mask,1)
     n2 = SIZE(mask,2)
     any_prefix_l2 = RESHAPE(prefix_l1(hpf_or_l1, &
          mask      = RESHAPE(mask, (/n/)), &
          segment   = RESHAPE(segment,(/n/)), &
          exclusive = exclusive), (/n1,n2/))
  ELSE 
     n = SIZE(mask)
     n1 = SIZE(mask,1)
     n2 = SIZE(mask,2)
     any_prefix_l2 = RESHAPE(prefix_l1(hpf_or_l1, &
          mask      = RESHAPE(mask, (/n/)), &
          exclusive = exclusive), (/n1,n2/))
  END IF

  RETURN
END FUNCTION any_prefix_l2
FUNCTION parity_prefix_l1 (mask, segment, exclusive)
  ! ..Use Definitions..
  USE hpf_func
  ! ..Implicit none ..
  IMPLICIT NONE
  ! ..Argument definitions..
  LOGICAL, INTENT(in) :: mask(:)
  LOGICAL, INTENT(in), OPTIONAL :: segment(:)
  LOGICAL, INTENT(in), OPTIONAL :: exclusive
  LOGICAL, DIMENSION(SIZE(mask)) :: parity_prefix_l1
  parity_prefix_l1 = prefix_l1 (hpf_neqv_l1, mask, segment, exclusive)

  RETURN
END FUNCTION parity_prefix_l1
FUNCTION parity_prefix_l2 (mask, dim, segment, exclusive)
  ! ..Use Definitions..
  USE hpf_func
  ! ..Implicit none ..
  IMPLICIT NONE
  ! ..Argument definitions..
  LOGICAL, INTENT(in) :: mask(:,:)
  INTEGER, INTENT(in), OPTIONAL :: dim
  LOGICAL, INTENT(in), OPTIONAL :: segment(:,:)
  LOGICAL, INTENT(in), OPTIONAL :: exclusive
  LOGICAL, DIMENSION(SIZE(mask,1),SIZE(mask,2)) :: parity_prefix_l2
  ! ..Local Variables..
  INTEGER :: n, n1, n2

  ! ..Executable Statements..
  IF (PRESENT(dim)) THEN
     parity_prefix_l2 = prefix_l2(hpf_neqv_l2, mask, dim, segment, &
          exclusive, identity = .false.)
  ELSE IF (PRESENT(segment)) THEN
     n = SIZE(mask)
     n1 = SIZE(mask,1)
     n2 = SIZE(mask,2)
     parity_prefix_l2 = RESHAPE(prefix_l1(hpf_neqv_l1, &
          mask      = RESHAPE(mask, (/n/)), &
          segment   = RESHAPE(segment,(/n/)), &
          exclusive = exclusive), (/n1,n2/))
  ELSE 
     n = SIZE(mask)
     n1 = SIZE(mask,1)
     n2 = SIZE(mask,2)
     parity_prefix_l2 = RESHAPE(prefix_l1(hpf_neqv_l1, &
          mask      = RESHAPE(mask, (/n/)), &
          exclusive = exclusive), (/n1,n2/))
  END IF

  RETURN
END FUNCTION parity_prefix_l2
FUNCTION prefix_l1 (func, mask, segment, exclusive, identity)
  ! ..Use definitions..
  USE errorfun 
  IMPLICIT NONE
  ! ..Arguments..
  ! ..Interface definition ..
  INTERFACE 
    FUNCTION func (A, B)
      USE precision
      LOGICAL, INTENT(in) ::  A(:), B(:)
      LOGICAL, DIMENSION(SIZE(A)) :: func
    END FUNCTION func
  END INTERFACE
  LOGICAL, INTENT(in) :: mask(:)
  LOGICAL, INTENT(in), OPTIONAL :: segment(:), exclusive, identity
  LOGICAL, DIMENSION(SIZE(mask)) :: prefix_l1
  ! .. Local Parameters ..
  CHARACTER(LEN=*), PARAMETER :: SRName = 'PREFIX'
  ! .. Local Variables ..
  LOGICAL, DIMENSION(SIZE(mask)) :: work
  LOGICAL, DIMENSION(SIZE(mask)) :: sm
  LOGICAL :: identity_element
  INTEGER :: istep, n

  ! Test for constraints on parameters
  n = SIZE(mask)
  IF (n == 0) THEN
     CALL error (100, SRName, 'MASK argument has size zero.')
     GOTO 999
  END IF
  IF (PRESENT(segment)) THEN
     IF (SIZE(segment) /= n) THEN
        CALL error (303, SRName, &
             'Argument SEGMENT does not have the same size as MASK.')
        GOTO 999
     END IF
  END IF
  IF (PRESENT(identity)) THEN
     identity_element = identity
  ELSE
     identity_element = .false.
  END IF

  work = mask
  ! Compute the segment mask
  IF (PRESENT(segment)) THEN
     sm = .not. (eoshift(segment,-1) .neqv. segment)
     sm(1) = .false.
     istep = 1
     DO
        IF (.not. ANY(sm)) EXIT
        ! if (istep >= n) exit ! This may be faster on some machines
        WHERE (sm(istep+1:n))
           work(istep+1:n) = func(work(istep+1:n),work(1:n-istep))
        END WHERE
        sm(istep+1:n) =  sm(istep+1:n) .and. sm(1:n-istep)
        istep = istep*2
     END DO
  ELSE
     istep = 1
     DO
        IF (istep >= n) EXIT
        work(istep+1:n) = func(work(istep+1:n),work(1:n-istep))
        istep = istep*2
     END DO
  END IF

  ! Exclusive?
  IF (PRESENT(exclusive)) THEN
     IF (exclusive) THEN
        work = eoshift(work,-1)
        IF (PRESENT(segment)) THEN
           sm = (eoshift(segment,-1) .neqv. segment)
           sm(1) = .true.
           WHERE (sm)
              work = identity_element
           END WHERE
        END IF
     END IF
  END IF

  prefix_l1 = work

  999 RETURN
END FUNCTION prefix_l1
FUNCTION prefix_l2 (func, mask, dim, segment, exclusive, identity)
  ! ..Use definitions..
  USE errorfun 
  IMPLICIT NONE
  ! ..Arguments..
  ! ..Interface definition ..
  INTERFACE 
    FUNCTION func (A, B)
      USE precision
      LOGICAL, INTENT(in) ::  A(:,:), B(:,:)
      LOGICAL, DIMENSION(SIZE(A,1),SIZE(A,2)) :: func
    END FUNCTION func
  END INTERFACE
  LOGICAL, INTENT(in) :: mask(:,:)
  INTEGER, INTENT(in) :: dim
  LOGICAL, INTENT(in), OPTIONAL :: segment(:,:), exclusive, identity
  LOGICAL, DIMENSION(SIZE(mask,1),SIZE(mask,2)) :: prefix_l2
  ! .. Local Parameters ..
  CHARACTER(LEN=*), PARAMETER :: SRName = 'PREFIX'
  ! .. Local Variables ..
  LOGICAL, DIMENSION(SIZE(mask,1),SIZE(mask,2)) :: work
  LOGICAL, DIMENSION(SIZE(mask,1),SIZE(mask,2)) :: sm
  INTEGER :: istep, n1, n2
  LOGICAL :: identity_element

  ! Test for constraints on parameters
  n1 = SIZE(mask,1)
  n2 = SIZE(mask,2)
  IF (SIZE(mask) == 0) THEN
     CALL error (100, SRName, &
          'ARRAY argument has size zero.')
     prefix_l2 = .false.
     GOTO 999
  END IF
  IF (PRESENT(segment)) THEN
     IF (SIZE(segment,1) /= n1 .or. &
          SIZE(segment,2) /= n2) THEN
        CALL error (303, SRName, &
             'Argument SEGMENT does not have the same size as ARRAY.')
        GOTO 999
     END IF
  END IF
  IF (dim>2) THEN
     CALL error (300, SRName, &
          'Argument DIM  is bigger than dimension of ARRAY.')
     GOTO 999
  END IF
  IF (dim<=0) THEN
     CALL error (301, SRName, &
          'Argument DIM must be bigger or equal to 1.')
     GOTO 999
  END IF
  IF (PRESENT(identity)) THEN
     identity_element = identity
  ELSE
     identity_element = .false.
  END IF

  work = mask
  ! Compute the segment mask
  IF (PRESENT(segment)) THEN
     IF (dim==1) THEN
        sm = .not. (eoshift(segment,-1,dim=dim) .neqv. segment)
        sm (1,:) = .false.
        istep = 1
        DO
           IF (.not. ANY(sm)) EXIT
           ! if (istep >= n) exit ! This may be faster on some machines
           WHERE (sm(istep+1:n1,:))
              work(istep+1:n1,:) = &
                   func(work(istep+1:n1,:),work(1:n1-istep,:))
           END WHERE
           sm(istep+1:n1,:) =  sm(istep+1:n1,:) .and. sm(1:n1-istep,:)
           istep = istep*2
        END DO
     ELSE
        sm = .not. (eoshift(segment,-1,dim=dim) .neqv. segment)
        sm(:,1) = .false.
        istep = 1
        DO
           IF (.not. ANY(sm)) EXIT
           ! if (istep >= n) exit ! This may be faster on some machines
           WHERE (sm(:,istep+1:n2))
              work(:,istep+1:n2) = &
                   func(work(:,istep+1:n2),work(:,1:n2-istep))
           END WHERE
           sm(:,istep+1:n2) =  sm(:,istep+1:n2) .and. sm(:,1:n2-istep)
           istep = istep*2
        END DO
     END IF
  ELSE
     IF (dim==1) THEN
        istep = 1
        DO
           IF (istep>=n1) EXIT
           work(istep+1:n1,:) = &
                func(work(istep+1:n1,:),work(1:n1-istep,:))
           istep = istep*2
        END DO
     ELSE                       ! dim=2
        istep = 1
        DO 
           IF (istep>=n2) EXIT
           work(:,istep+1:n2) = &
                func(work(:,istep+1:n2),work(:,1:n2-istep))
           istep = istep*2
        END DO
     END IF
  END IF

  ! Exclusive?
  IF (PRESENT(exclusive)) THEN
     IF (exclusive) THEN
        IF (PRESENT(segment)) THEN
           sm = (eoshift(segment,-1,dim=dim) .neqv. segment)
           IF (dim==1) THEN
              sm(1,:) = identity_element
           ELSE
              sm(:,1) = identity_element
           endif
           work = eoshift(work,-1,dim=dim)
           WHERE (sm)
              work = identity_element
           END WHERE
        ELSE
           work = eoshift(work,-1,dim=dim,boundary=identity_element)
        END IF
     END IF
  END IF

  prefix_l2 = work

  999 RETURN
END FUNCTION prefix_l2
END MODULE HPF_PREFIX_L
