MODULE HPF_PREFIX_S

  USE Precision

CONTAINS
FUNCTION copy_prefix_s1 (array, segment)
  USE errorfun 
  USE hpf_func
  IMPLICIT NONE
  REAL(kind=sp), INTENT(in) :: array(:)
  LOGICAL, INTENT(in), OPTIONAL :: segment(:)
  REAL(kind=sp), DIMENSION(SIZE(array)) :: copy_prefix_s1
  copy_prefix_s1 = prefix_s1 (hpf_copy_s1, array, segment=segment)

  RETURN
END FUNCTION copy_prefix_s1
FUNCTION copy_prefix_s2 (array, dim, segment)
  ! ..Use Definitions..
  USE errorfun 
  USE hpf_func
  ! ..Implicit none ..
  IMPLICIT NONE
  ! ..Argument definitions..
  REAL(kind=sp), INTENT(in) :: array(:,:)
  INTEGER, INTENT(in), OPTIONAL :: dim
  LOGICAL, INTENT(in), OPTIONAL :: segment(:,:)
  REAL(kind=sp), DIMENSION(SIZE(array,1),SIZE(array,2)) :: copy_prefix_s2
  ! ..Local Variables..
  INTEGER :: n, n1, n2

  ! ..Executable Statements..
  IF (PRESENT(dim)) THEN
     COPY_PREFIX_S2 = PREFIX_S2(hpf_copy_s2, array, dim, &
          segment = segment)
  ELSE IF (PRESENT(segment)) THEN
     n = SIZE(array)
     n1 = SIZE(array,1)
     n2 = SIZE(array,2)
     COPY_PREFIX_S2 = RESHAPE(prefix_s1(hpf_copy_s1, &
          array     = RESHAPE(array, (/n/)), &
          segment   = RESHAPE(segment,(/n/))), (/n1,n2/))
  ELSE 
     n = SIZE(array)
     n1 = SIZE(array,1)
     n2 = SIZE(array,2)
     COPY_PREFIX_S2 = RESHAPE(prefix_s1(hpf_copy_s1, &
          array     = RESHAPE(array, (/n/))), (/n1,n2/))
  END IF

  RETURN
END FUNCTION copy_prefix_s2
FUNCTION MAXVAL_prefix_s1 (array, mask, segment, exclusive)
  USE hpf_func
  IMPLICIT NONE
  REAL(kind=sp), INTENT(in) :: array(:)
  LOGICAL, INTENT(in), OPTIONAL :: mask(:), segment(:), exclusive
  REAL(kind=sp), DIMENSION(SIZE(array)) :: MAXVAL_prefix_s1
  ! ..Local Variables..
  REAL(sp) :: identity



  identity = MINVAL(array)



  ! ..Executable Statements..
  MAXVAL_prefix_s1 = prefix_s1(hpf_MAX_s1, array, mask, &
     segment, exclusive, identity = identity)

  RETURN
END FUNCTION MAXVAL_prefix_s1
  FUNCTION MAXVAL_prefix_s2 (array, dim, mask, segment, exclusive)
    USE hpf_func
    IMPLICIT NONE
    REAL(kind=sp), INTENT(in) :: array(:,:)
    INTEGER, INTENT(in), OPTIONAL :: dim
    LOGICAL, INTENT(in), OPTIONAL :: mask(:,:), segment(:,:), exclusive
    REAL(kind=sp), DIMENSION(SIZE(array,1),SIZE(array,2)) :: MAXVAL_prefix_s2
  ! ..Local Variables..
  REAL(sp) :: identity
  INTEGER :: n, n1, n2



  identity = MINVAL(array)



  ! ..Executable Statements..
  IF (PRESENT(dim)) THEN
     MAXVAL_prefix_s2 = prefix_s2(hpf_MAX_s2, array, dim, mask, &
          segment, exclusive, identity = identity)
  ELSE IF (PRESENT(mask) .and. PRESENT(segment)) THEN
     n = SIZE(array)
     n1 = SIZE(array,1)
     n2 = SIZE(array,2)
     MAXVAL_prefix_s2 = RESHAPE(prefix_s1(hpf_MAX_s1, &
          array     = RESHAPE(array, (/n/)), &
          mask      = RESHAPE(mask, (/n/)), &
          segment   = RESHAPE(segment,(/n/)), &
          exclusive = exclusive, &
          identity   = identity), (/n1,n2/))
  ELSE IF (PRESENT(mask)) THEN
     n = SIZE(array)
     n1 = SIZE(array,1)
     n2 = SIZE(array,2)
     MAXVAL_prefix_s2 = RESHAPE(prefix_s1(hpf_MAX_s1, &
          array     = RESHAPE(array, (/n/)), &
          mask      = RESHAPE(mask, (/n/)), &
          exclusive = exclusive, &
          identity   = identity), (/n1,n2/))
  ELSE IF (PRESENT(segment)) THEN
     n = SIZE(array)
     n1 = SIZE(array,1)
     n2 = SIZE(array,2)
     MAXVAL_prefix_s2 = RESHAPE(prefix_s1(hpf_MAX_s1, &
          array     = RESHAPE(array, (/n/)), &
          segment   = RESHAPE(segment,(/n/)), &
          exclusive = exclusive, &
          identity   = identity), (/n1,n2/))
  ELSE 
     n = SIZE(array)
     n1 = SIZE(array,1)
     n2 = SIZE(array,2)
     MAXVAL_prefix_s2 = RESHAPE(prefix_s1(hpf_MAX_s1, &
          array     = RESHAPE(array, (/n/)), &
          exclusive = exclusive, &
          identity   = identity), (/n1,n2/))
  END IF

  RETURN
END FUNCTION MAXVAL_prefix_s2
  FUNCTION MINVAL_prefix_s1 (array, mask, segment, exclusive)
    USE hpf_func
    IMPLICIT NONE
    REAL(kind=sp), INTENT(in) :: array(:)
    LOGICAL, INTENT(in), OPTIONAL :: mask(:), segment(:), exclusive
    REAL(kind=sp), DIMENSION(SIZE(array)) :: MINVAL_prefix_s1
  ! ..Local Variables..
  REAL(sp) :: identity



  identity = MAXVAL(array) - 1


  MINVAL_prefix_s1 = prefix_s1 (hpf_MIN_s1, array, mask, segment, &
     exclusive, identity)

  RETURN
END FUNCTION MINVAL_prefix_s1
  FUNCTION MINVAL_prefix_s2 (array, dim, mask, segment, exclusive)
    USE hpf_func
    IMPLICIT NONE
    REAL(kind=sp), INTENT(in) :: array(:,:)
    INTEGER, INTENT(in), OPTIONAL :: dim
    LOGICAL, INTENT(in), OPTIONAL :: mask(:,:), segment(:,:), exclusive
    REAL(kind=sp), DIMENSION(SIZE(array,1),SIZE(array,2)) :: MINVAL_prefix_s2
  ! ..Local Variables..
  REAL(sp) :: identity
  INTEGER :: n, n1, n2



  identity = MAXVAL(array)



  ! ..Executable Statements..
  IF (PRESENT(dim)) THEN
     MINVAL_prefix_s2 = prefix_s2(hpf_MIN_s2, array, dim, mask, &
          segment, exclusive, identity = identity)
  ELSE IF (PRESENT(mask) .and. PRESENT(segment)) THEN
     n = SIZE(array)
     n1 = SIZE(array,1)
     n2 = SIZE(array,2)
     MINVAL_prefix_s2 = RESHAPE(prefix_s1(hpf_MIN_s1, &
          array     = RESHAPE(array, (/n/)), &
          mask      = RESHAPE(mask, (/n/)), &
          segment   = RESHAPE(segment,(/n/)), &
          exclusive = exclusive, &
          identity   = identity), (/n1,n2/))
  ELSE IF (PRESENT(mask)) THEN
     n = SIZE(array)
     n1 = SIZE(array,1)
     n2 = SIZE(array,2)
     MINVAL_prefix_s2 = RESHAPE(prefix_s1(hpf_MIN_s1, &
          array     = RESHAPE(array, (/n/)), &
          mask      = RESHAPE(mask, (/n/)), &
          exclusive = exclusive, &
          identity   = identity), (/n1,n2/))
  ELSE IF (PRESENT(segment)) THEN
     n = SIZE(array)
     n1 = SIZE(array,1)
     n2 = SIZE(array,2)
     MINVAL_prefix_s2 = RESHAPE(prefix_s1(hpf_MIN_s1, &
          array     = RESHAPE(array, (/n/)), &
          segment   = RESHAPE(segment,(/n/)), &
          exclusive = exclusive, &
          identity   = identity), (/n1,n2/))
  ELSE 
     n = SIZE(array)
     n1 = SIZE(array,1)
     n2 = SIZE(array,2)
     MINVAL_prefix_s2 = RESHAPE(prefix_s1(hpf_MIN_s1, &
          array     = RESHAPE(array, (/n/)), &
          exclusive = exclusive, &
          identity   = identity), (/n1,n2/))
  END IF

  RETURN
END FUNCTION MINVAL_prefix_s2
FUNCTION product_prefix_s1 (array, mask, segment, exclusive)
  USE hpf_func
  IMPLICIT NONE
  REAL(kind=sp), INTENT(in) :: array(:)
  LOGICAL, INTENT(in), OPTIONAL :: mask(:), segment(:), exclusive
  REAL(kind=sp), DIMENSION(SIZE(array)) :: product_prefix_s1
  product_prefix_s1 = prefix_s1 (hpf_mult_s1, array, mask, segment, &
     exclusive, 1.0_sp)

    RETURN
  END FUNCTION product_prefix_s1
FUNCTION product_prefix_s2 (array, dim, mask, segment, exclusive)
  USE hpf_func
  IMPLICIT NONE
  REAL(kind=sp), INTENT(in) :: array(:,:)
  INTEGER, INTENT(in), OPTIONAL :: dim
  LOGICAL, INTENT(in), OPTIONAL :: mask(:,:), segment(:,:), exclusive
  REAL(kind=sp), DIMENSION(SIZE(array,1),SIZE(array,2)) :: product_prefix_s2
  ! ..Local Variables..
  INTEGER :: n, n1, n2

  ! ..Executable Statements..
  IF (PRESENT(dim)) THEN
     product_prefix_s2 = prefix_s2(hpf_mult_s2, array, dim, mask, &
          segment, exclusive, identity = 1.0_sp)
  ELSE IF (PRESENT(mask) .and. PRESENT(segment)) THEN
     n = SIZE(array)
     n1 = SIZE(array,1)
     n2 = SIZE(array,2)
     product_prefix_s2 = RESHAPE(prefix_s1(hpf_mult_s1, &
          array     = RESHAPE(array, (/n/)), &
          mask      = RESHAPE(mask, (/n/)), &
          segment   = RESHAPE(segment,(/n/)), &
          exclusive = exclusive, &
          identity   = 1.0_sp), (/n1,n2/))
  ELSE IF (PRESENT(mask)) THEN
     n = SIZE(array)
     n1 = SIZE(array,1)
     n2 = SIZE(array,2)
     product_prefix_s2 = RESHAPE(prefix_s1(hpf_mult_s1, &
          array     = RESHAPE(array, (/n/)), &
          mask      = RESHAPE(mask, (/n/)), &
          exclusive = exclusive, &
          identity   = 1.0_sp), (/n1,n2/))
  ELSE IF (PRESENT(segment)) THEN
     n = SIZE(array)
     n1 = SIZE(array,1)
     n2 = SIZE(array,2)
     product_prefix_s2 = RESHAPE(prefix_s1(hpf_mult_s1, &
          array     = RESHAPE(array, (/n/)), &
          segment   = RESHAPE(segment,(/n/)), &
          exclusive = exclusive, &
          identity   = 1.0_sp), (/n1,n2/))
  ELSE 
     n = SIZE(array)
     n1 = SIZE(array,1)
     n2 = SIZE(array,2)
     product_prefix_s2 = RESHAPE(prefix_s1(hpf_mult_s1, &
          array     = RESHAPE(array, (/n/)), &
          exclusive = exclusive, &
          identity   = 1.0_sp), (/n1,n2/))
  END IF

  RETURN
END FUNCTION product_prefix_s2
FUNCTION sum_prefix_s1 (array, mask, segment, exclusive)
  USE hpf_func
  IMPLICIT NONE
  REAL(kind=sp), INTENT(in) :: array(:)
  LOGICAL, INTENT(in), OPTIONAL :: mask(:), segment(:), exclusive
  REAL(kind=sp), DIMENSION(SIZE(array)) :: sum_prefix_s1
    ! Call generic prefix routine
    sum_prefix_s1 = prefix_s1 (hpf_add_s1, array, mask, segment, exclusive)

    RETURN
  END FUNCTION sum_prefix_s1
FUNCTION sum_prefix_s2 (array, dim, mask, segment, exclusive)
  ! ..Use Definitions..
  USE hpf_func
  ! ..Implicit none ..
  IMPLICIT NONE
  REAL(kind=sp), INTENT(in) :: array(:,:)
  INTEGER, INTENT(in), OPTIONAL :: dim
  LOGICAL, INTENT(in), OPTIONAL :: mask(:,:), segment(:,:), exclusive
  REAL(kind=sp), DIMENSION(SIZE(array,1),SIZE(array,2)) :: sum_prefix_s2
  ! ..Local Variables..
  INTEGER :: n, n1, n2

  ! ..Executable Statements..
  IF (PRESENT(dim)) THEN
     sum_prefix_s2 = prefix_s2(hpf_add_s2, array, dim, mask, &
          segment, exclusive, identity = 0.0_sp)
  ELSE IF (PRESENT(mask) .and. PRESENT(segment)) THEN
     n = SIZE(array)
     n1 = SIZE(array,1)
     n2 = SIZE(array,2)
     sum_prefix_s2 = RESHAPE(prefix_s1(hpf_add_s1, &
          array     = RESHAPE(array, (/n/)), &
          mask      = RESHAPE(mask, (/n/)), &
          segment   = RESHAPE(segment,(/n/)), &
          exclusive = exclusive, &
          identity   = 0.0_sp), (/n1,n2/))
  ELSE IF (PRESENT(mask)) THEN
     n = SIZE(array)
     n1 = SIZE(array,1)
     n2 = SIZE(array,2)
     sum_prefix_s2 = RESHAPE(prefix_s1(hpf_add_s1, &
          array     = RESHAPE(array, (/n/)), &
          mask      = RESHAPE(mask, (/n/)), &
          exclusive = exclusive, &
          identity   = 0.0_sp), (/n1,n2/))
  ELSE IF (PRESENT(segment)) THEN
     n = SIZE(array)
     n1 = SIZE(array,1)
     n2 = SIZE(array,2)
     sum_prefix_s2 = RESHAPE(prefix_s1(hpf_add_s1, &
          array     = RESHAPE(array, (/n/)), &
          segment   = RESHAPE(segment,(/n/)), &
          exclusive = exclusive, &
          identity   = 0.0_sp), (/n1,n2/))
  ELSE 
     n = SIZE(array)
     n1 = SIZE(array,1)
     n2 = SIZE(array,2)
     sum_prefix_s2 = RESHAPE(prefix_s1(hpf_add_s1, &
          array     = RESHAPE(array, (/n/)), &
          exclusive = exclusive, &
          identity   = 0.0_sp), (/n1,n2/))
  END IF

  RETURN
END FUNCTION sum_prefix_s2
FUNCTION prefix_s1 (func, array, mask, segment, exclusive, identity)
  ! ..Use definitions..
  USE errorfun 
  IMPLICIT NONE
  ! ..Arguments..
  ! ..Interface definition ..
  INTERFACE 
    FUNCTION func (A, B)
      USE precision
      REAL(sp), INTENT(in) ::  A(:), B(:)
      REAL(sp), DIMENSION(SIZE(A)) :: func
    END FUNCTION func
  END INTERFACE
  REAL(kind=sp), INTENT(in) :: array(:)
  LOGICAL, INTENT(in), OPTIONAL :: mask(:), segment(:), exclusive
  REAL(kind=sp), INTENT(in), OPTIONAL :: identity
  REAL(kind=sp), DIMENSION(SIZE(array)) :: prefix_s1
  ! .. Local Parameters ..
  CHARACTER(LEN=*), PARAMETER :: SRName = 'PREFIX'
  ! .. Local Variables ..
  REAL(kind=sp), DIMENSION(SIZE(array)) :: work
  REAL :: identity_element
  LOGICAL, DIMENSION(SIZE(array)) :: sm
  INTEGER :: istep, n

  ! Test for constraints on parameters
  n = SIZE(array)
  IF (n == 0) THEN
     CALL error (100, SRName, &
          'ARRAY argument has size zero.')
     prefix_s1 = 0
     GOTO 999
  END IF
  IF (PRESENT(mask)) THEN
     IF (SIZE(mask) /= n) THEN
        CALL error (302, SRName, &
             'Argument MASK does not have the same size as ARRAY.')
        GOTO 999
     END IF
  END IF
  IF (PRESENT(segment)) THEN
     IF (SIZE(segment) /= n) THEN
        CALL error (303, SRName, &
             'Argument SEGMENT does not have the same size as ARRAY.')
        GOTO 999
     END IF
  END IF
  IF (PRESENT(identity)) THEN
     identity_element = identity
  ELSE
     identity_element = 0.0
  END IF

  ! Is there a MASK or not?
  IF (PRESENT(mask)) THEN
     WHERE (mask)
        work = array
     ELSEWHERE
        work = identity_element
     END WHERE
  ELSE      
     work = array
  END IF

  ! Compute the segment mask
  IF (PRESENT(segment)) THEN
     sm = .not. (eoshift(segment,-1) .neqv. segment)
     sm(1) = .false.
     istep = 1
     DO
        IF (.not. ANY(sm(1:n))) EXIT
        ! if (istep >= n) exit ! This may be faster on some machines
        WHERE (sm(istep+1:n))
           work(istep+1:n) = func(work(istep+1:n),work(1:n-istep))
        END WHERE
        sm(istep+1:n) =  sm(istep+1:n) .and. sm(1:n-istep)
        istep = istep*2
     END DO
  ELSE
     istep = 1
     DO
        IF (istep >= n) EXIT
        work(istep+1:n) = func(work(istep+1:n),work(1:n-istep))
        istep = istep*2
     END DO
  END IF

  ! Exclusive?
  IF (PRESENT(exclusive)) THEN
     IF (exclusive) THEN
        IF (PRESENT(segment)) THEN
           sm = (eoshift(segment,-1) .neqv. segment)
           sm(1) = .true.
           work = eoshift(work,-1)
           WHERE (sm)
              work = 0.0
           END WHERE
        ELSE
           work = eoshift(work,-1)
        END IF
     END IF
  END IF

  prefix_s1 = work

  999 RETURN
END FUNCTION prefix_s1
FUNCTION prefix_s2 (func, array, dim, mask, segment, exclusive, identity)
  ! ..Use definitions..
  USE errorfun
  IMPLICIT NONE
  ! ..Arguments..
  ! ..Interface definition ..
  INTERFACE 
    FUNCTION func (A, B)
      USE precision
      REAL(sp), INTENT(in) ::  A(:,:), B(:,:)
      REAL(sp), DIMENSION(SIZE(A,1),SIZE(A,2)) :: func
    END FUNCTION func
  END INTERFACE
  REAL(kind=sp), INTENT(in) :: array(:,:)
  INTEGER, INTENT(in) :: dim
  LOGICAL, INTENT(in), OPTIONAL :: mask(:,:), segment(:,:), exclusive
  REAL(kind=sp), INTENT(in), OPTIONAL :: identity
  REAL(kind=sp), DIMENSION(SIZE(array,1),SIZE(array,2)) :: prefix_s2
  ! .. Local Parameters ..
  CHARACTER(LEN=*), PARAMETER :: SRName = 'PREFIX'
  ! .. Local Variables ..
  REAL(kind=sp), DIMENSION(SIZE(array,1),SIZE(array,2)) :: work
  LOGICAL, DIMENSION(SIZE(array,1),SIZE(array,2)) :: sm
  REAL :: identity_element
  INTEGER :: n1, n2, istep

  ! Test for constraints on parameters
  n1 = SIZE(array,1)
  n2 = SIZE(array,2)
  IF (SIZE(array) == 0) THEN
     CALL error (100, SRName, &
          'ARRAY argument has size zero.')
     prefix_s2 = 0
     GOTO 999
  END IF
  IF (dim>2) THEN
     CALL error (300, SRName, &
          'Argument DIM  is bigger than dimension of ARRAY.')
     GOTO 999
  END IF
  IF (dim<=0) THEN
     CALL error (301, SRName, &
          'Argument DIM must be bigger or equal to 1.')
     GOTO 999
  END IF
  IF (PRESENT(mask)) THEN
     IF (SIZE(mask,1) /= SIZE(array,1) .or. &
          SIZE(mask,2) /= SIZE(array,2)) THEN
        CALL error (302, SRName, &
             'Argument MASK does not have the same size as ARRAY.')
   	GOTO 999
     END IF
  END IF
  IF (PRESENT(segment)) THEN
     IF (SIZE(segment,1) /= SIZE(array,1) .or. &
          SIZE(segment,2) /= SIZE(array,2)) THEN
        CALL error (303, SRName, &
             'Argument SEGMENT does not have the same size as ARRAY.')
   	GOTO 999
     END IF
  END IF
  IF (PRESENT(identity)) THEN
     identity_element = identity
  ELSE
     identity_element = 0.0
  END IF

  ! Is there a MASK or not?
  IF (PRESENT(mask)) THEN
     WHERE (mask)
        work = array
     ELSEWHERE
        work = identity_element
     END WHERE
  ELSE      
     work = array
  END IF

  ! Compute the segment mask
  IF (PRESENT(segment)) THEN
     IF (dim==1) THEN
        sm = .not. (eoshift(segment,-1,dim=dim) .neqv. segment)
        sm (1,:) = .false.
        istep = 1
        DO
           IF (.not. ANY(sm)) EXIT
           ! This may be faster on some machines
           ! if (istep >= n) exit 
           WHERE (sm(istep+1:n1,:))
              work(istep+1:n1,:) = &
                   func(work(istep+1:n1,:),work(1:n1-istep,:))
           END WHERE
           sm(istep+1:n1,:) =  sm(istep+1:n1,:) .and. sm(1:n1-istep,:)
           istep = istep*2
        END DO
     ELSE
        sm = .not. (eoshift(segment,-1,dim=dim) .neqv. segment)
        sm(:,1) = .false.
        istep = 1
        DO
           IF (.not. ANY(sm)) EXIT
           ! if (istep >= n) exit ! This may be faster on some machines
           WHERE (sm(:,istep+1:n2))
              work(:,istep+1:n2) = &
                   func(work(:,istep+1:n2),work(:,1:n2-istep))
           END WHERE
           sm(:,istep+1:n2) =  sm(:,istep+1:n2) .and. sm(:,1:n2-istep)
           istep = istep*2
        END DO
     END IF
  ELSE
     IF (dim==1) THEN
        istep = 1
        DO
           IF (istep>=n1) EXIT
           work(istep+1:n1,:) = &
                func(work(istep+1:n1,:),work(1:n1-istep,:))
           istep = istep*2
        END DO
     ELSE                       ! dim=2
        istep = 1
        DO 
           IF (istep>=n2) EXIT
           work(:,istep+1:n2) = &
                func(work(:,istep+1:n2),work(:,1:n2-istep))
           istep = istep*2
        END DO
     END IF
  END IF

  ! Exclusive?
  IF (PRESENT(exclusive)) THEN
     IF (exclusive) THEN
        IF (PRESENT(segment)) THEN
           sm = (eoshift(segment,-1,dim=dim) .neqv. segment)
           work = eoshift(work,-1,dim=dim)
           WHERE (sm)
              work = 0.0
           END WHERE
        ELSE
           work = eoshift(work,-1,dim=dim)
        END IF
     END IF
  END IF

  prefix_s2 = work

  999 CONTINUE
  RETURN
END FUNCTION prefix_s2
END MODULE HPF_PREFIX_S
