MODULE HPF_SCATTER_L

CONTAINS
FUNCTION ScatterWith_l1_i1(func,identity,array,base,indx1,mask)
  ! ..Use Definitions..
  USE HPF_sort
  USE HPF_prefix
  USE precision
  ! ..Implicit None..
  IMPLICIT None
  ! ..Mandatory Arguments..
  INTERFACE
    FUNCTION func(A,B)
      USE precision
      LOGICAL, intent(in) :: A(:) , B(:)
      LOGICAL, dimension(size(A)) :: func
    END FUNCTION func
  END INTERFACE

  LOGICAL, intent(in)                   :: identity
  LOGICAL, intent(in)                   :: array(:)
  LOGICAL, intent(in)                   :: base(:)
  INTEGER, intent(in), dimension(size(array)) :: indx1
  
  ! ..Optional Arguments..
  LOGICAL, intent(in), optional               :: mask(:)
  
  ! ..Function Result..
  LOGICAL, dimension(lbound(base,dim=1): &
                           ubound(base,dim=1))::ScatterWith_l1_i1
  ! ..Local Parameters..
  CHARACTER(len=*), parameter :: SRName = 'Scatter_l1_i1'
  ! ..Local Variables..
  LOGICAL, dimension(lbound(array,dim=1): &
                           ubound(array,dim=1)):: dummy_array
  INTEGER, dimension(lbound(array,dim=1):ubound(array,dim=1)):: dummy_indx
  LOGICAL, dimension(lbound(array,dim=1): &
                           ubound(array,dim=1)):: working_set
  INTEGER, dimension(lbound(array,dim=1):ubound(array,dim=1)):: perm_ivec
  LOGICAL, dimension(lbound(array,dim=1):ubound(array,dim=1)):: segment_mask
  LOGICAL, dimension(lbound(base,dim=1): &
                           ubound(base,dim=1)):: working_base
  LOGICAL, dimension(lbound(base,dim=1):ubound(base,dim=1)):: base_mask
 
  INTEGER :: low_arr , hi_arr
  IF (minval(indx1) < lbound(base,dim=1)) THEN
     call error (302, SRName, &
                 'Scatter to a location below the lower bound of the array')
  ENDIF
  IF (maxval(indx1) > ubound(base,dim=1)) THEN
     print '(A,I6,A,I6)' , 'Lower bound = ', lbound(indx1,dim=1) , &
                           'Upper bound = ', ubound(indx1,dim=1)
     print * , indx1
     call error (303, SRName, &
                 'Scatter to a location above the upper bound of the array')
  ENDIF
  IF (present(mask)) THEN
    low_arr = lbound(array,dim=1)
    hi_arr  = low_arr + count(mask,DIM=1) -1
    dummy_array(low_arr:hi_arr)= pack(array,mask,dummy_array(low_arr:hi_arr))
    dummy_indx(low_arr:hi_arr) = pack(indx1,mask,dummy_indx(low_arr:hi_arr))
  ELSE
    low_arr = lbound(array,dim=1)
    hi_arr  = ubound(array,dim=1)
    dummy_array = array
    dummy_indx  = indx1
  ENDIF
  working_set(:) = identity
  perm_ivec(low_arr:hi_arr)= GRADE_UP_I1(dummy_indx(low_arr:hi_arr))
  working_set(:)= dummy_array(perm_ivec(:))
  dummy_indx(:) = dummy_indx(perm_ivec(:))
  segment_mask          = dummy_indx /= eoshift(dummy_indx,shift=-1,dim=1)
  segment_mask(low_arr) = .True.

  segment_mask          = parity_prefix_l1(segment_mask)
  working_set           = prefix_l1(func,working_set,   &
				    segment=segment_mask,identity=identity)

  segment_mask = segment_mask .neqv. eoshift(segment_mask,shift=1,dim=1)
  segment_mask(hi_arr) = .True.

  working_base = identity
  base_mask = .false.
  WHERE (segment_mask)
    working_base(dummy_indx) = working_set
    base_mask(dummy_indx) = .true.
  ENDWHERE
  WHERE (base_mask)
     ScatterWith_l1_i1 = func(base,working_base)
  ELSEWHERE
     ScatterWith_l1_i1 = base
  ENDWHERE

END FUNCTION ScatterWith_l1_i1
FUNCTION ScatterWith_l2_i2(func,identity,array,base,indx1,indx2,mask)
  ! ..Use Definitions..
  USE HPF_sort
  USE HPF_prefix
  USE precision
  ! ..Implicit None..
  IMPLICIT None
  ! ..Mandatory Arguments..
  INTERFACE
    FUNCTION func(A,B)
      USE precision
      LOGICAL, intent(in) :: A(:) , B(:)
      LOGICAL, dimension(size(A)) :: func
    END FUNCTION func
  END INTERFACE

  LOGICAL, intent(in)                      :: identity
  LOGICAL, intent(in)                      :: array(:,:)
  LOGICAL, intent(in)                      :: base(:,:)
  INTEGER, intent(in),                              &
           dimension(size(array,1),size(array,2)):: indx1(:,:) , indx2(:,:)
  
  ! ..Optional Arguments..
  LOGICAL, intent(in),                              &
           dimension(size(array,1), size(array,2)), &
           optional                              :: mask(:,:)
  
  ! ..Function Result..
  LOGICAL, dimension(lbound(base,dim=1):ubound(base,dim=1), &
                     lbound(base,dim=2):ubound(base,dim=2))::ScatterWith_l2_i2
  ! ..Local Parameters..
  CHARACTER(len=*), parameter :: SRName = 'Scatter_l2_i2'
  IF (present(mask)) THEN
    ScatterWith_l2_i2                                              &
      = reshape( ScatterWith_l1_i1(                                &
                   func,                                           &
                    identity,                                      &
                    reshape(array,shape=(/size(array)/)),          &
                    reshape(base ,shape=(/size(base)/)),           &
                    reshape(( ((indx2-1) * (size(indx1,dim=1))) +  &
                              indx1                             +  &
                              (lbound(indx1,dim=1)-1)),            &
                            (/size(indx1)/)),                      &
                    mask=reshape(mask,shape=(/size(mask)/))),      &
                 (/size(base,dim=1),size(base,dim=2)/))
  ELSE
    ScatterWith_l2_i2                                              &
      = reshape( ScatterWith_l1_i1(                                &
                   func,                                           &
                   identity,                                       &
                   reshape(array,shape=(/size(array)/)),           &
                   reshape(base ,shape=(/size(base)/)),            &
                   reshape(( ((indx2-1) * (size(indx1,dim=1))) +   &
                             indx1                             +   &
                             (lbound(indx1,dim=1)-1)),             &
                           (/size(indx1)/))),                      &
                 (/size(base,dim=1),size(base,dim=2)/))
  END IF
END FUNCTION ScatterWith_l2_i2
FUNCTION ScatterWith_l1_i2(func,identity,array,base,indx1,mask)
  ! ..Use Definitions..
  USE HPF_sort
  USE HPF_prefix
  ! ..Implicit None..
  IMPLICIT None
  ! ..Mandatory Arguments..
  INTERFACE
    FUNCTION func(A,B)
      USE precision
      LOGICAL, intent(in) :: A(:) , B(:)
      LOGICAL, dimension(size(A)) :: func
    END FUNCTION func
  END INTERFACE

  LOGICAL, intent(in)                      :: identity
  LOGICAL, intent(in)                      :: array(:,:)
  LOGICAL, intent(in)                      :: base(:)
  INTEGER, intent(in),                              &
           dimension(size(array,1),size(array,2)):: indx1(:,:) 
  
  ! ..Optional Arguments..
  LOGICAL, intent(in),                              &
           dimension(size(array,1), size(array,2)), &
           optional                              :: mask(:,:)
  
  ! ..Function Result..
  LOGICAL, dimension(lbound(base,dim=1): &
                           ubound(base,dim=1))::ScatterWith_l1_i2
  ! ..Local Parameters..
  CHARACTER(len=*), parameter :: SRName = 'Scatter_l1_i2'
  IF (present(mask)) THEN
    ScatterWith_l1_i2                                     &
      = ScatterWith_l1_i1(                                &
           func,                                          &
           identity,                                      &
           reshape(array,shape=(/size(array)/)),          &
           base,                                          &
           reshape(indx1,(/size(indx1)/)),                &
           mask=reshape(mask,shape=(/size(mask)/)))
  ELSE
    ScatterWith_l1_i2                                     &
      = ScatterWith_l1_i1(                                &
           func,                                          & 
           identity,                                      &
           reshape(array,shape=(/size(array)/)),          &
           base ,                                         &
           reshape(indx1,(/size(indx1)/)))
  END IF
END FUNCTION ScatterWith_l1_i2

FUNCTION ScatterWith_l2_i1(func,identity,array,base,indx1,indx2,mask)
  ! ..Use Definitions..
  USE HPF_sort
  USE HPF_prefix
  ! ..Implicit None..
  IMPLICIT None
  ! ..Mandatory Arguments..
  INTERFACE
    FUNCTION func(A,B)
      USE precision
      LOGICAL, intent(in) :: A(:) , B(:)
      LOGICAL, dimension(size(A)) :: func
    END FUNCTION func
  END INTERFACE

  LOGICAL, intent(in)       :: identity
  LOGICAL, intent(in)       :: array(:)
  LOGICAL, intent(in)       :: base(:,:)
  INTEGER, intent(in),             &
           dimension(size(array)) :: indx1(:) , indx2(:)
  
  ! ..Optional Arguments..
  LOGICAL, intent(in),             &
           dimension(size(array)), &
           optional               :: mask(:)
  
  ! ..Function Result..
  LOGICAL, dimension(lbound(base,dim=1):ubound(base,dim=1), &
                     lbound(base,dim=2):ubound(base,dim=2))::ScatterWith_l2_i1
  ! ..Local Parameters..
  CHARACTER(len=*), parameter :: SRName = 'Scatter_l2_i1'
  IF (present(mask)) THEN
    ScatterWith_l2_i1                                              &
      = reshape( ScatterWith_l1_i1(                                &
                    func,                                          &
                    identity,                                      &
                    array,                                         &
                    reshape(base ,shape=(/size(base)/)),           &
                    (  ((indx2-1) * (size(indx1,dim=1))) +         &
                       indx1                             +         &
                       (lbound(indx1,dim=1)-1)),                   &
                    mask=mask),                                    &
                 (/size(base,dim=1),size(base,dim=2)/))
  ELSE
    ScatterWith_l2_i1                                              &
      = reshape( ScatterWith_l1_i1(                                &
                    func,                                          &
                    identity,                                      &
                    array,                                         &
                    reshape(base ,shape=(/size(base)/)),           &
                    (  ((indx2-1) * (size(indx1,dim=1))) +         &
                       indx1                             +         &
                       (lbound(indx1,dim=1)-1))),                  &
                 (/size(base,dim=1),size(base,dim=2)/))
  END IF
END FUNCTION ScatterWith_l2_i1
END MODULE HPF_SCATTER_L
