MODULE HPF_Sort
  USE precision, only : wp=>sp
  USE ErrorFun
  
  logical success
  integer, private:: i
  
  public radixsort
  
  INTERFACE GRADE_UP
    MODULE PROCEDURE GRADE_UP_I1
  END INTERFACE

  INTERFACE GRADE_DOWN
    MODULE PROCEDURE GRADE_DOWN_I1
  END INTERFACE
  
  private
  
CONTAINS
FUNCTION GRADE_UP_I1(array, dim)
  ! ..Mandatory Arguments..
  INTEGER, intent(IN)             :: array(:)
  ! ..Optional  Arguments
  INTEGER, intent(IN), optional   :: dim
  ! ..Function Result..
  INTEGER, dimension(size(array)) :: GRADE_UP_I1
  ! .. Local Parameters ..
  CHARACTER(len=*), parameter :: SRName = 'GRADE_UP'
  INTEGER i

   IF (present(dim)) THEN
     IF (dim > 1) THEN
        call error (301, SRName , &
                    'Argument DIM is larger than rank of the array.')
     ENDIF
   ENDIF
   GRADE_UP_I1 = RadixSort(array,(/2,2,2,2,2,2,2,2,2,3,3,4,4/),.True.)
   GRADE_UP_I1(GRADE_UP_I1(:)) = (/(i,i=lbound(array,DIM=1), &
                                        ubound(array,DIM=1))/)
END FUNCTION GRADE_UP_I1
FUNCTION GRADE_DOWN_I1(array, dim)
  ! ..Mandatory Arguments..
  INTEGER, intent(IN)             :: array(:)
  ! ..Optional  Arguments
  INTEGER, intent(IN), optional   :: dim
  ! ..Function Result..
  INTEGER, dimension(size(array)) :: GRADE_DOWN_I1
  ! .. Local Parameters ..
  character(len=*), parameter :: SRName = 'GRADE_DOWN'

   IF (present(dim)) THEN
     IF (dim > 1) THEN
        call error (301, SRName , &
                    'Argument DIM is larger than rank of the array.')
     ENDIF
   ENDIF
   GRADE_DOWN_I1 = RadixSort(array,(/2,2,2,2,2,2,2,2,2,3,3,4,4/),.False.)
   GRADE_DOWN_I1(GRADE_DOWN_I1(:)) = (/(i,i=lbound(array,DIM=1), &
                                            ubound(array,DIM=1))/)
END FUNCTION GRADE_DOWN_I1
FUNCTION RadixSort(unsorted,radix,ascend)
  ! ..Mandatory Arguments..
  INTEGER, intent(IN)                :: unsorted(:)
  INTEGER, intent(IN)                :: radix(:)
  LOGICAL, intent(IN)                :: ascend
  ! ..Function Result..
  INTEGER, dimension(size(unsorted)) :: RadixSort
  ! ..Local Parameters..
  CHARACTER(len=*), parameter :: SRName = 'RadixSort'
  ! ..Local Variables..
  INTEGER, dimension(1+size(radix))   :: scanned_radix
  INTEGER, dimension(lbound(unsorted,dim=1):                      &
                     ubound(unsorted,dim=1)):: permute_unsorted , &
                                               permute_ivec     , &
                                               window_unsorted
  INTEGER :: i
  LOGICAL :: already_sorted
  IF (sum(radix)/=32) THEN
    call error(300, SRName, ' Radix sort requires 32-bit radix')
  ENDIF
  scanned_radix(1) = 0
  DO i=1,size(radix)
    scanned_radix(i+1) = scanned_radix(i) + radix(i)
  END DO
  WHERE (btest(unsorted,31)) 
     permute_unsorted = ibclr(unsorted,31) 
  ELSEWHERE 
     permute_unsorted = ibset(unsorted,31) 
  END WHERE
  permute_ivec = (/(i,i=lbound(unsorted,DIM=1), &
                        ubound(unsorted,DIM=1))/) 

  DO i=1,size(radix)
     window_unsorted = GetBits(permute_unsorted,    & 
                               scanned_radix(i),  &
                               scanned_radix(i+1)-1)

     IF (ascend) THEN
        already_sorted = all (window_unsorted >=       &
                              eoshift(window_unsorted, SHIFT=-1))
     ELSE
        already_sorted = all (window_unsorted >=       &
                              eoshift(window_unsorted, SHIFT=1))
     ENDIF

     IF (already_sorted) THEN
        ! Do absolutely nothing, this chunk is already sorted
     ELSE
        IF (radix(i) > 2) THEN
          window_unsorted = ItemsBefore(window_unsorted,         &
                                        minval(window_unsorted), &
                                        maxval(window_unsorted), &
                                        ascend)
        ELSE
          window_unsorted = ItemsBefore(window_unsorted,0,       &
                                        2**radix(i), ascend)
        ENDIF
        
        permute_ivec(:) = window_unsorted(permute_ivec(:))
        IF (i < size(radix)) THEN
          permute_unsorted(window_unsorted(:)) = permute_unsorted(:)
        ENDIF
     ENDIF
  ENDDO
  RadixSort = permute_ivec
END FUNCTION RadixSort
FUNCTION GetBits(vec,from,to)  
  ! ..Mandatory Arguments..
  INTEGER, intent(IN)           :: vec(:)
  INTEGER, intent(IN)           :: from, to
  ! ..Function Result..
  INTEGER, dimension(size(vec)) :: GetBits
  ! ..Local Variables..
  INTEGER                       :: i
  INTEGER, dimension(size(vec)) :: temp

  temp = 0
  DO i=0,(to-from)
    WHERE (btest(vec,from+i)) temp = ibset(temp,i)
  END DO
  GetBits = temp
END FUNCTION GetBits
FUNCTION ItemsBefore (unsorted,minUnsorted,maxUnsorted,ascend)
  ! ..Implicit None..
  IMPLICIT None

  ! ..Mandatory Arguments..
  INTEGER, intent(in)               :: unsorted(:)
  INTEGER, intent(in)               :: minUnsorted, maxUnsorted
  LOGICAL, intent(in)               :: ascend

  ! ..Function Result..
  INTEGER, dimension(size(unsorted)):: ItemsBefore
  ! ..Local Variables..
  INTEGER, DIMENSION(size(unsorted)) :: accum_res, temp_res



  LOGICAL, DIMENSION(size(unsorted)) :: hits

  INTEGER :: i , j ,offset, flagged , sort_size , sort_lower, sort_upper
  INTEGER :: start_bucket , end_bucket, step_bucket
  sort_size  = size(unsorted)
  sort_lower = lbound(unsorted,dim=1)
  sort_upper = ubound(unsorted,dim=1) 

  IF (ascend) THEN
    start_bucket = minUnsorted
    end_bucket   = maxUnsorted
    step_bucket  = 1
  ELSE
    start_bucket = maxUnsorted
    end_bucket   = minUnsorted
    step_bucket  = -1
  ENDIF
  offset = sort_lower-1



  DO i   = start_bucket , end_bucket, step_bucket
     hits(:) = unsorted(:) == i
     flagged = count(hits)
     IF (flagged == 0) THEN
       ! Do nothing
     ELSE IF (flagged == 1) THEN
       WHERE (hits) accum_res = offset + 1
       offset = offset + 1
     ELSE


        WHERE (hits)
           temp_res = 1
        ELSEWHERE
           temp_res = 0
        END WHERE
        j=1
        DO WHILE (j < sort_size .AND. temp_res(j) < flagged)
           temp_res(j+1:sort_size) = temp_res(j+1:sort_size) + &
                                     temp_res(1:sort_size-j)
           j = 2*j
        END DO


        WHERE (hits) accum_res = temp_res + offset
        offset = offset + flagged
     ENDIF
  END DO
  ItemsBefore = accum_res
END FUNCTION ItemsBefore

END MODULE HPF_Sort
