package BOutilities.BObrowser;

import backoffice.BOgui.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.*;

public class Browser extends Frame
{
	final static private int COLUMNS= 3;
	final static private String 
		FILE= "File",
		ABOUT= "About...",
		EXIT= "Close";
		
	private BrowserNode root;
	private BrowserNodeViewFactory factory;
	
	private int columns;
	private BrowserNodeListView [] listViews;
	private BrowserNodeContentView  contentView;
	
	private Frame parent;
	private Frame browserFrame;
		
	public Browser(String title, Frame parent) {
		this(title, null, COLUMNS);
		this.parent= parent;
		browserFrame= this;
	}
			
	public Browser(String title, BrowserNodeViewFactory factory, int columns) {
		super(title);
		
		this.root= null;
		this.factory= factory;
		this.columns= columns;
		listViews= new BrowserNodeListView[columns];
	}
	
	protected void initialize(BrowserNodeViewFactory factory) {
		
		this.factory= factory;
		
		this.makeMenuBar();
		this.makeContent();
		this.installEventHandling();
		
		setSize(500, 500);
		
		pack();
	}
		
	public void setRoot(BrowserNode node) {
		root= node;
		listViews[0].replaceBrowserNodes(node.getChildren());
	}	
	
	public void aboutClose() {}
	
	private void makeMenuBar() {
		MenuBar b= new MenuBar();
		Menu m= new Menu(FILE, true);
		MenuItem i= new MenuItem(ABOUT);
		
		ActionListener show= new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				new AboutDialog(browserFrame);
			}
		};
		i.addActionListener(show);
		
		m.add(i);
		m.addSeparator();

		i= new MenuItem(EXIT);
		
		ActionListener quitter= new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				aboutClose();
				dispose();
			}
		};
		i.addActionListener(quitter);
		
		m.add(i);

		b.add(m);
		setMenuBar(b);
	}
	
	
	private void installEventHandling() {
		WindowListener wl= new WindowAdapter() {
			public void windowClosing(WindowEvent event) {
				aboutClose();
				dispose();
			}
		};
		
		addWindowListener(wl);
		
		if(parent != null)
			parent.addWindowListener(wl);
	}
	
	protected void setTitles(BrowserNodeListView [] listViews) {}
	
	private void makeContent() {
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints constraint = new GridBagConstraints();
	
		setLayout(gridbag);
		
		constraint.fill = GridBagConstraints.BOTH;
		constraint.weightx = 1.0;
		constraint.weighty = 1.0;
		constraint.gridwidth =  1;    
		constraint.gridheight = 1;

		// browser part
		for(int i= 0; i < columns; i++) {
			if(i == columns -1)
				constraint.gridwidth = GridBagConstraints.REMAINDER;
			listViews[i]= factory.createBrowserNodeListView();
			gridbag.setConstraints(listViews[i], constraint);
			add(listViews[i]);
		}
		
		for(int i= 1; i < columns; i++)
			listViews[i].setParent(listViews[i - 1]);
		
		setTitles(listViews);
		
		// content part
		constraint.gridwidth = GridBagConstraints.REMAINDER;
		contentView= factory.createBrowserNodeContentView();
		gridbag.setConstraints(contentView, constraint);
		add(contentView);
		
		for(int i= 0; i < columns; i++)
			contentView.observe(listViews[i]);
		
	}		
}
