package BOutilities.BObrowser;

import BOutilities.BOcanvas.*;

import java.awt.*;
import java.awt.event.*;
import java.util.Vector;

public class BrowserNodeListView extends Panel implements ItemListener
{
	static final int LENGTH= 5;
	
	// view
	List        list;
	TextCanvas  label;
	BrowserNodeListView  parent;
	
	// model
	Vector    browserNodes;
	
	public BrowserNodeListView() {
		this("", null);
	}
	
	public BrowserNodeListView(String title) {
		this(title, null);
	}
	
	public BrowserNodeListView(String title, BrowserNodeListView parent) {
		
		this.parent= null;
		this.label= new TextCanvas(title);
		this.list= new List(LENGTH);
		this.browserNodes= new Vector();
		
		makeContent();
		
		setParent(parent);
	}
	
	private void makeContent() {
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints constraint = new GridBagConstraints();
	
		setLayout(gridbag);
		
		constraint.gridwidth = GridBagConstraints.REMAINDER;
		
		gridbag.setConstraints(this.label, constraint);
		add(this.label);
		gridbag.setConstraints(this.list, constraint);
		add(this.list);
	}
		
	public void setParent(BrowserNodeListView parent) {
		removeParentDependencies();
		this.parent= parent;
		installParentDependencies();
	}
	
	private void removeParentDependencies() {
		for(BrowserNodeListView p= this.parent; p != null; p= p.parent)
			p.getList().removeItemListener(this);
	}
	
	private void installParentDependencies() {
		for(BrowserNodeListView p= this.parent; p != null; p= p.parent)
			p.getList().addItemListener(this);
	}		
		
	public void setTitle(String title) { 
		label.setText(title); 
	}
	
	public List getList() {
		return list;
	}
	
	public BrowserNode getNodeAt(int index) {
		return (BrowserNode) browserNodes.elementAt(index);
	}
	
	public void addBrowserNode(BrowserNode node) {
		browserNodes.addElement(node);
		list.add(node.toBrowserString());
	}
	
	public void removeBrowserNode(BrowserNode node) {
		int i= browserNodes.indexOf(node);
		browserNodes.removeElementAt(i);
		list.remove(i);
	}
	
	public void removeAllBrowserNodes() {
		list.removeAll();
		browserNodes.removeAllElements();
	}
	
	public void replaceBrowserNodes(BrowserNode [] nodes) {
		removeAllBrowserNodes();
		
		for(int i= 0; i < nodes.length; i++) 
			addBrowserNode(nodes[i]);
	}
				
	public void itemStateChanged(ItemEvent e) {
		if(e.getItemSelectable() == parent.getList()) {
			int index= ((Integer) e.getItem()).intValue();
			BrowserNode selected= parent.getNodeAt(index);
			replaceBrowserNodes(selected.getChildren());
		} else
			removeAllBrowserNodes();
	}
	
//	public Dimension getMinimumSize() {
//		return getPreferredSize();
//	}
//	
//	public Dimension getPreferredSize() {
//		return new Dimension(150, 150);
//	}
}
