package backoffice.BOkernel;

import java.util.*;

public class BOScheduler extends Object implements Observer
{
	private BackOffice backOffice;
	private Vector     processors;
	
	public BOScheduler(BackOffice backOffice) {
		this.backOffice= backOffice;
		this.processors= new Vector();
	}
	
	
	// configuration interface 
	
	public void addProcessor(BOProcessor processor) {
		synchronized(processors) {
			processors.addElement(processor);
		}
	}
	
	public void removeProcessor(BOProcessor processor) {
		synchronized(processors) {
			processors.removeElement(processor);
		}
	}
	
	
	// observer interface + scheduler actions
	
	public void update(Observable o, Object arg) {
		schedule((Date) arg);
	}
                                            
	private void schedule(Date now) {
		// find mature deals
		BODealbase dealbase= backOffice.dealbase();
		Vector deals= dealbase.checkoutMatureDeals(now);
		if(deals == null) return;
		
		// System.out.println("****" + now.toString() + "****");
		
		// dispatch deals according to their first transaction
		Enumeration e= deals.elements();
		while(e.hasMoreElements()) {
			BODeal deal= (BODeal) e.nextElement();
			BOTransaction transaction= deal.nextTransactionForProcessing();
			BOProcessor p= dispatch(transaction);
			if(p != null)
				p.process(transaction);
		}
	}
	
	private BOProcessor dispatch(BOTransaction transaction) {
		Enumeration e = processors.elements();
		while(e.hasMoreElements()) {
			BOProcessor p= (BOProcessor) e.nextElement();
			if(p.canProcess(transaction))
				return backOffice.factory().createProcessor(p, backOffice);
		} 
		return null;
	}
			
}
