package backoffice.BOkernel;

import backoffice.BOutil.*;
import backoffice.BOgui.*;
import BOutilities.BObrowser.*;

import java.util.*;

public class BackOffice extends Object implements BrowserNode
{
	protected TradingLine tradingLine;
	protected BackOfficeEnvironment environment;
	
	protected BODealbase dealbase;
	protected BOArchive  archive;
	protected BOScheduler scheduler;
	
	protected BackOfficeFrame presentation;
	
	public BackOffice(TradingLine tradingLine, BackOfficeEnvironment environment) {
		this.tradingLine= tradingLine;
		this.environment= environment;
		
		environment.bankSystem().enroll(this);
	}
	
	public void run() {		
		initialize();
		presentation= new BackOfficeFrame(this, "Back Office");
		presentation.show();
	}
	
	protected void initialize() {
		BackOfficeFactory f= environment.backOfficeFactory();
		dealbase=  f.createDealbase(this);
		archive=   f.createArchive(this);
		scheduler= f.createScheduler(this);
		environment.subscribeTimeBeat(scheduler);
	}
	
	public TradingLine tradingLine() {
		return tradingLine;
	}
	
	public BackOfficeFactory factory() {
		return environment.backOfficeFactory(); 
	}
	
	public BOArchive archive() {
		return archive;
	}
	
	public BODealbase dealbase() {
		return dealbase;
	}
	
	public BOScheduler scheduler() {
		return scheduler;
	}
	
	public BackOfficeEnvironment environment() {
		return environment;
	}
				
	public void acceptDeal(BODeal deal) {
		dealbase.addDeal(deal);
	}
	
	public void quit() {
		// remove from book keeping
		environment.bankSystem().remove(this);
		// close presentation
		presentation.dispose(); 
		presentation= null;
	}
			
	
	// interface BrowserNode
	
	public BrowserNode [] getChildren() {
		BrowserNode [] children = { dealbase().asBrowserNode(), archive().asBrowserNode() };
		return children;
	}
	
	public BrowserNode getContent() {
		return null;
	}
	
	public boolean hasContent() {
		return false;
	}
	
	public String toBrowserString() {
		return toString();
	}

}
