package backoffice.BOkernel;

import backoffice.BOgui.*;

import java.awt.event.*;
import java.util.*;

public class FrontOffice extends Object
{
	// model
	private FrontOfficeEnvironment environment;
	private String ident;
	private Vector deals;
	private BackOffice backOffice;
	
	// view
	private FrontOfficeFrame presentation;
	
	public FrontOffice(String ident, FrontOfficeEnvironment environment) {
		this.ident= ident;
		this.environment= environment;
		this.deals= new Vector();
		this.backOffice= null;
		
		environment.bankSystem().enroll(this);
	}
	
	public String ident() {
		return ident;
	}
	
	public void run() {
		presentation= new FrontOfficeFrame(this, ident);
		presentation.show();
	}

	public void quit() {
		// submit remaining deals
		if(backOffice != null) submitDeals();
		// remove from book keeping
		environment.bankSystem().remove(this);
		// close presentation
		presentation.dispose(); 
		presentation= null;
	}	
		
	public ActionListener createListener() {
		return new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				BODeal deal=environment.frontOfficeFactory().createDeal();
				deals.addElement(deal);
				presentation.addDeal(deal);
			}
		};
	}	
	
	public void connectTo(BackOffice backOffice) {
		this.backOffice= backOffice;
	}
	
	public ActionListener connectListener() {
		return new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				BankSystem system= environment.bankSystem();
				system.connect(FrontOffice.this);
			}
		};
	}	
	
	void submitDeals() {
		// deals might be changed as soon as accepted by back office
		// since the presentation relies on string representation, everything
		// must be completed before accept
		 
		if(backOffice == null) 
		{
			System.out.println("No existing connection to BackOffice");
			return;
		}
		
		BODeal [] toSubmit;
		synchronized(deals) {
			toSubmit= new BODeal[deals.size()];
			deals.copyInto(toSubmit);
			deals.removeAllElements();
		}
		presentation.removeAllDeals();
		
		for(int i= 0; i < toSubmit.length; i++)
			backOffice.acceptDeal(toSubmit[i]);
	}
	
	public ActionListener submitListener() {
		return new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				submitDeals();
			}
		};
	}				
}
