package backoffice.BOsimulation;

import java.util.*;

public class SimulationClock extends Observable implements Runnable
{
	private static final int   WAIT= 1000;
	private static final long  INC=  86400000;
	
	private boolean quit;
	private Date date;
	private long increment;
	private Thread beat;
	
	public SimulationClock() {
		this.quit= false;
		this.date= new Date();
		this.increment= INC;
		this.beat= null;
	}
	
	
	// clock implementation
	
	public void run() {
		for(;;) {
			synchronized(this) {
				try { wait(WAIT); } catch(InterruptedException e) {}
				if(quit) break;
			}
			setChanged();
			notifyObservers(nextDate());
		}
		beat.stop();
	}
	
	
	// clock manipulation interface
	
	synchronized public void quit() {
		this.quit= true;
		notifyAll();
	}
	
	synchronized public void start() {
		if(beat == null) {
			beat= new Thread(this);
			beat.start();
		} else
			beat.resume();
	}
	
	synchronized public void stop() {
		if(beat != null)
			beat.suspend();
	}
	
	synchronized public void reset() {
		date= new Date(INC);
	}			
		
	synchronized public void incrementTime() {
		increment += INC;
	}

	synchronized public void decrementTime() {
		if(increment > INC)
			increment -= INC;
	}

	
	// clock query interface
	
	synchronized public Date now() {
		return date;
	}	
	
	public int incrementInDays() {
		return (int) (increment / INC);
	}
	
	
	
	// private implementation 
	
	private Date nextDate() {
		date.setTime(date.getTime() + increment);
		return date;
	}
	
}
