package backoffice.BOsimulation;

import BOutilities.BOcanvas.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import java.text.*;

class ClockDisplay extends TextCanvas implements Observer
{
	final static private String TIME= "Time: ";
	final static private DateFormat dateFormat= DateFormat.getDateInstance(DateFormat.SHORT);

	ClockDisplay(SimulationClock clock) {
		super("");
		clock.addObserver(this);
		setText(TIME + dateFormat.format(clock.now()));
	}
	
	public void update(Observable o, Object arg) {
		setText(TIME + dateFormat.format((Date) arg));
	}
}

class TimeIncrement extends TextCanvas
{
	final static private String  START= "Time Increment ", END= " day(s)";
	SimulationClock clock;
		
	TimeIncrement(SimulationClock clock) {
		super("");
		this.clock= clock;
		setText(START + clock.incrementInDays() + END);
	}
	
	void update() {
		setText(START + clock.incrementInDays() + END);
		repaint();
	}
}

public class SimulationClockControl extends Panel 
{
	private SimulationClock clock;
	private TimeIncrement increment;
	
	public SimulationClockControl(SimulationClock clock) {
		this.clock= clock;
		makeContent();
	}
	
	private void makeContent() {
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints constraint = new GridBagConstraints();
	
		setLayout(gridbag);
		
		constraint.fill= GridBagConstraints.BOTH;
		constraint.weightx= 1.0;
		constraint.weighty= 1.0;
		constraint.insets= new Insets(0,5,0,5);

		constraint.gridheight= 2;       
		constraint.gridwidth=  GridBagConstraints.REMAINDER;   
		
		Component component= new TextCanvas("Simulation Control");
		gridbag.setConstraints(component, constraint);		
		add(component);
		
		constraint.gridheight= 1;
		constraint.gridwidth= 1;
		component= new ClockDisplay(clock);
		gridbag.setConstraints(component, constraint);		
		add(component);
		
		constraint.gridwidth= GridBagConstraints.REMAINDER;;    
		increment= new TimeIncrement(clock);
		gridbag.setConstraints(increment, constraint);		
		add(increment);
		
		constraint.gridwidth= 1;
		Button button= new Button("Start");
		gridbag.setConstraints(button, constraint);		
		add(button);
		button.addActionListener(startListener());
		
		constraint.gridwidth= GridBagConstraints.REMAINDER;
		button= new Button("Increment by 24h");
		gridbag.setConstraints(button, constraint);		
		add(button); 
		button.addActionListener(incrementListener());
		
		constraint.gridwidth= 1;
		button= new Button("Stop");
		gridbag.setConstraints(button, constraint);		
		add(button); 
		button.addActionListener(stopListener());
		
		constraint.gridwidth= GridBagConstraints.REMAINDER;
		button= new Button("Decrement by 24h");
		gridbag.setConstraints(button, constraint);		
		add(button); 
		button.addActionListener(decrementListener());
							
		setSize(200,120);
	}
	
	public ActionListener incrementListener() {
		return new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				clock.incrementTime();
				increment.update();
				
			}
		};
	}
		
	public ActionListener decrementListener() {
		return new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				clock.decrementTime();
				increment.update();
				
			}
		};
	}
	
	public ActionListener startListener() {
		return new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				clock.start();
			}
		};
	}
	
	public ActionListener stopListener() {
		return new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				clock.stop();				
			}
		};
	}
}
