package backoffice.BOutil;

import backoffice.BOkernel.*;
import BOutilities.BObrowser.*;

import java.util.*;
import java.text.*;

class DealStoreEntry implements BrowserNode
{

	private Day day;
	private DealStore dealStore;
	
	private static final DateFormat dateFormat= DateFormat.getDateInstance(DateFormat.SHORT);
	
	DealStoreEntry(Day day, DealStore dealStore) {
		this.day= day;
		this.dealStore= dealStore;
	}
	
	
	public BrowserNode [] getChildren() {
		Vector deals= dealStore.dealsFor(day);
		int size= deals.size();
		BrowserNode [] children= new BrowserNode[size];
		for(int i= 0; i < size; i++) 
			children[i] = (BrowserNode) deals.elementAt(i);
		
		return children;
	}

	public String toBrowserString() {
		return dateFormat.format(day);
	}
	
	public boolean hasContent() {
		return false;
	}

	public BrowserNode getContent() {
		return this;
	}
}

public class DealStore {

	/*	Deals stored in a DealStore must not be changed! 
	 *	Deals which have to be changed must be removed in advance.
	 *  The DealStore handles finshed and unfinished deals differently.
	 */
	 
	private String       title;
	private Hashtable    store;
	private DateSelector selector;
	 
	public DealStore(String title, DateSelector selector) {
		this.title= title;
		this.selector= selector;
		this.store= new Hashtable();
	}	
	
	public void addDeal(BODeal deal) {
		doAddDeal(deal, selector.date(deal));
	}
	
	public void removeDeal(BODeal deal) {
		doRemoveDeal(deal, selector.date(deal));
	}
	
	public Vector dealsBefore(Date date) {
		Day day= new Day(date);
		Vector v1= getDealsBefore(day);
		Vector v2= getDealsOnBefore(day, date);
		appendVector(v1, v2);
		return v1;
	}
	
	public BrowserNode asBrowserNode() {
		return new DealNode();
	}
	
	Vector dealsFor(Day day) {
		return doDealsFor(day);
	}
		
		
		
	// private methods 
	
	private void doAddDeal(BODeal deal, Date where) {
		Day key= new Day(where);
		Vector v= (Vector) store.get(key);
		if(v == null) {
			v= new Vector();
			v.addElement(deal);
		} else 
			insertDeal(deal, v);	
		store.put(key, v);
	}
	
	
	private void doRemoveDeal(BODeal deal, Date where) {
		Day key= new Day(where);
		Vector v= (Vector) store.get(key);
		if( v != null) {
			while(v.removeElement(deal)) ;
			if(v.size() == 0)
				store.remove(key);
		}
	}
	
	private Vector doDealsFor(Day day) {
		Vector v= (Vector) store.get(day);
		if(v == null) 
			v= new Vector();
		return v;
	}
	
	private void insertDeal(BODeal deal, Vector v) {
		int index= -1;
		Enumeration e= v.elements();
		while(e.hasMoreElements()) {
			BODeal d= (BODeal) e.nextElement();
			if(selector.date(deal).before(selector.date(d))) {
				index= v.indexOf(d);
				break;
			}
		}
		if(index == -1)
			v.addElement(deal);
		else
			v.insertElementAt(deal, index);
	}
	
	private Vector getDealsBefore(Day day) {
		Vector result= new Vector();
		Enumeration e= store.keys();
		while(e.hasMoreElements()) {
			Day d= (Day) e.nextElement();
			if(d.before(day))
				appendVector(result, (Vector) store.get(d));
		}
		return result;
	}
				
	private  Vector getDealsOnBefore(Day day, Date date) {
		Vector result= (Vector) store.get(day);
		if(result == null) 
			return new Vector();
		Enumeration	e= result.elements();
		BODeal d= (BODeal) e.nextElement();
		while(selector.date(d).before(date)) {
			result.addElement(d);
			if(!e.hasMoreElements()) break;
		}
		return result;
	}
			
	private void appendVector(Vector target, Vector source) {
		Enumeration e= source.elements();
		while(e.hasMoreElements())
			target.addElement(e.nextElement());
	}
	
		
	private class DealNode implements BrowserNode
	{		
		public BrowserNode [] getChildren() {
			int size= store.size();
			
			BrowserNode [] entries= new BrowserNode[size];
			
			Enumeration e= store.keys();
			for(int i = 0; i < size; i++) {
				Day day= (Day) e.nextElement();
				entries[i]= new DealStoreEntry(day, DealStore.this);
			}
			
			return entries;
		}
		
		public String toBrowserString() {
			return title;
		}
		
		public boolean hasContent() {
			return false;
		}
		
		public BrowserNode getContent() {
			return this;
		}
	}

}
	
	
		
