package backoffice;

import backoffice.BOkernel.*;
import backoffice.BOutil.TradingLine;
import backoffice.BOtest.*;
import backoffice.BOsimulation.*;

import java.util.*;
import java.applet.*;
import java.awt.event.*;

public class BackOfficeApplication extends Applet
{
	final static boolean DEBUG= true;
	public boolean inApplet= true;
	
	// view
	private BackOfficeApplicationFrame presentation;
	
	// model
	private OfficeSimulationEnvironment environment;
	
	public BackOfficeApplication() {
		super();
		
		this.presentation= null;

		this.environment= new OfficeSimulationEnvironment();
		this.environment.setBankSystem(new BankSystem(new TestLineManager()));
		this.environment.setBackOfficeFactory(new BackOfficeTestFactory());
		this.environment.setFrontOfficeFactory(new FrontOfficeTestFactory(environment));
	}

	public void init() {
		try {
			(new BackOfficeApplication()).run();
	    } catch (Exception e) {
		    e.printStackTrace();
		};
	}

	void run() {
		inApplet= false;
		presentation= new BackOfficeApplicationFrame(this, "Back Office System");
		presentation.show();
	}
	
	public static void main(String args[]) {
		for (int i= 0; i < (args.length - 1); i += 2) {
			System.out.println(args[i] + " = " + args[i + 1]);
		}

		(new BackOfficeApplication()).run();
	}
	
	OfficeSimulationEnvironment getEnvironment() {
		return environment;
	}
	
	void quit() {
		presentation.dispose(); 
		presentation= null;
		
		environment.simulationClock().quit();
		
		if(inApplet) {
			stop();
			destroy();
		} 
		else
			System.exit(0);
	}
	
	// it's only here to speed up the compiler
	void startFrontOffice() {
		if(DEBUG) System.out.println("FrontOffice started");
		(new FrontOffice(generateTitle("FrontOffice"), environment)).run();
	}
	
	ActionListener frontOfficeListener() {
		return new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				startFrontOffice();
			}
		};
	}
	
	// it's only here to speed up the compiler
	void startBackOffice() {
		if(DEBUG) System.out.println("BackOffice started");
		TradingLine tl= new TradingLine(generateTitle("BackOffice"));
		(new BackOffice(tl, environment)).run();
	}
	
	ActionListener backOfficeListener() {
		return new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				startBackOffice();
			}
		};
	}
	
	private String generateTitle(String s) {
		final Hashtable map= new Hashtable();
		
		Object val= map.get(s);
		int counter= 0;
		if(val != null)
			counter= ((Integer) val).intValue();
		map.put(s, new Integer(++counter));
		return (s + counter);
	}
		
}
