package backoffice;

import backoffice.BOgui.*;
import backoffice.BOkernel.*;
import backoffice.BOsimulation.*;
import BOutilities.BOcanvas.*;

import java.awt.*;
import java.awt.event.*;

public class BackOfficeApplicationFrame extends Frame 
{
	final static private String 
		FILE= "File",
		FRONT= "Front Office",
		BACK= "Back Office",
		ABOUT= "About...",
		EXIT= "Exit";
							
	private BackOfficeApplication application;
	private BackOfficeApplicationFrame applicationFrame;
		
	public BackOfficeApplicationFrame(BackOfficeApplication app, String title) {
		super(title);
		application= app;
		applicationFrame= this;
		
		makeMenuBar();
		makeContent();
	}
	
	private void makeMenuBar() {
		MenuBar b= new MenuBar();
		Menu m= new Menu(FILE, true);
		MenuItem i= new MenuItem(ABOUT);
		
		ActionListener show= new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				new AboutDialog(applicationFrame);
			}
		};
		i.addActionListener(show);
		
		m.add(i);
		m.addSeparator();

		i= new MenuItem(EXIT);
		
		ActionListener quitter= new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				application.quit();
			}
		};
		i.addActionListener(quitter);
		
		m.add(i);

		b.add(m);
		setMenuBar(b);
	}
	
	private void makeContent() {
			
		WindowListener wl= new WindowAdapter() {
			public void windowClosing(WindowEvent event) {
				application.quit();
			}
		};
		this.addWindowListener(wl);
		
		
		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints constraint = new GridBagConstraints();
	
		setLayout(gridbag);
		
		constraint.fill = GridBagConstraints.BOTH;
		constraint.weightx = 1.0;
		constraint.weighty = 1.0;
		constraint.gridwidth =  2;    
		constraint.gridheight = 2;

		Component component= new ImageCanvas("./BOgui/BOimages/backoffice.jpg");
		gridbag.setConstraints(component, constraint);
		add(component);
		
		constraint.gridwidth = GridBagConstraints.REMAINDER;
		constraint.gridheight = 1;       
		
		Button button= new Button(FRONT);
		gridbag.setConstraints(button, constraint);		
		add(button);
		button.addActionListener(application.frontOfficeListener());
		
		button= new Button(BACK);
		gridbag.setConstraints(button, constraint);		
		add(button); 
		button.addActionListener(application.backOfficeListener());
		
		constraint.gridheight = 4;       
		component= new SimulationClockControl(application.getEnvironment().simulationClock());
		gridbag.setConstraints(component, constraint);
		add(component);	
		
		setSize(300,150);
		
		pack();
	}	
}

